/*******************************************************
 NAME:		KF_Equations.h
 AUTHOR:	Luis Montesano
 DATE:		06-04-04
 DESCRIPTION:
	Contains the header of the kalman filter implementation
	for tracking people (or moving objects) based on xy 
	observations of the positions of the objects.

********************************************************/

#ifndef KF_EQUATIONS_h
#define KF_EQUATIONS_h

#include <stdio.h>
#include "sp_matrix.h"
#include "sp_matrix.c"

#include "imported/TData.h"

/* DATA TYPES DEFINITIONS */
typedef struct {	
	VECTOR x;
	MATRIX P;
} Tkf;


/* FUNCTION DEFINITIONS */

void initKFModule(float s2vx, float s2vy, float os2x, float os2y);
/* Initializes the kf module (motion and measurement models) */

void initializeKF(Tpf const *pos, Tkf *kf);
/* Creates a new kalman filter in the given position */

void predKFPT(Tkf const *inkf, float period, Tkf *predkf);
/* Kalman filter prediction step */

void updateKFPT(Tkf const *predkf, Tpf const *z, Tkf *updkf);
/* Kalman filter update step */

int moveKF(Tsc const *motion,  Tkf const *kfs, Tkf *movedkfs);
/* uncertain motion */

void printFilter(Tkf *kf, char *msg);
/* print data on the screen */

#endif
