/***************************************************/
/* Last Revised:
$Id: Bresenham.c,v 1.1 2004/02/11 20:29:28 jminguez Exp jminguez $
*/
/***************************************************/

#include <stdlib.h>
#include <stdio.h>

#include "Matriz.h"
#include "Bresenham.h"


/* ---------------------------------------------------------------------*/
/* ---------------------------------------------------------------------*/
/*  Funciones externas                                                  */
/* ---------------------------------------------------------------------*/
/* ---------------------------------------------------------------------*/

void LineBresenham(int x1,int y1,int x2,int y2,
		   float a[][DIMMATRIZ], short tag){

  /* Fill the projected line onto matrix */
  int deltax,deltay,numpixels;
  int d,dinc1,dinc2,xinc1,xinc2,yinc1,yinc2;
  int x,y,i;

  deltax = abs(x2 - x1);
  deltay = abs(y2 - y1);

  /*  Initialize all vars based on which is the independent variable  */

  if (deltax >= deltay){
    /* x is independent variable  */
    numpixels = deltax + 1;
    d = (deltay<<1) - deltax;
    dinc1 = deltay<<1;
    dinc2 = (deltay - deltax)<<1;
    xinc1 = 1;
    xinc2 = 1;
    yinc1 = 0;
    yinc2 = 1;
  }
  else{
    /* y is independent variable */
    numpixels = deltay + 1;
    d = (deltax<<1) - deltay;
    dinc1 = deltax<<1;
    dinc2 = (deltax - deltay)<<1;
    xinc1 = 0;
    xinc2 = 1;
    yinc1 = 1;
    yinc2 = 1;
  }
 
  /* Make sure x and y move in the right directions  */
  if (x1 > x2){
    xinc1 = - xinc1;
    xinc2 = - xinc2;
  }
 
  if (y1 > y2){
    yinc1 = - yinc1;
    yinc2 = - yinc2;
  }

  /*  Start drawing at <x1, y1>  */
  x = x1;
  y = y1;

  
  /*  Draw the pixels  */
  for (i = 1; i< numpixels; i++){
    /* antes de actualizar miro si se sale fuera para parar */
/*     if (x<0 || y<0 || x>=DIMMATRIZ || y>=DIMMATRIZ)  */
/*       return 0; */
/*     else */
    a[x][y]=a[x][y]+VACIO;
    if (d < 0){
      d = d + dinc1;
      x = x + xinc1;
      y = y + yinc1;
    }
    else{
      d = d + dinc2;
      x = x + xinc2;
      y = y + yinc2;
    }
  } /*fin del bucle*/        

  //if (tag==SPT) a[x][y]=a[x][y]-VACIO+LLENO;
  //else if (tag==UPT) a[x][y]=a[x][y]-VACIO;
  if (tag==SPT) a[x][y]=a[x][y]+LLENO;
  else if (tag==MPT) a[x][y]=a[x][y]+VACIO;
}



int RayTracing(int x1,int y1,int x2,int y2,
		   float a[][DIMMATRIZ], int *xout, int *yout){

  /* Follow the trajectory of the ray until the first */
  /* occupied cell */
  int deltax,deltay,numpixels;
  int d,dinc1,dinc2,xinc1,xinc2,yinc1,yinc2;
  int x,y,i;
  int res=0;

  deltax = abs(x2 - x1);
  deltay = abs(y2 - y1);

  /*  Initialize all vars based on which is the independent variable  */

  if (deltax >= deltay){
    /* x is independent variable  */
    numpixels = deltax + 1;
    d = (deltay<<1) - deltax;
    dinc1 = deltay<<1;
    dinc2 = (deltay - deltax)<<1;
    xinc1 = 1;
    xinc2 = 1;
    yinc1 = 0;
    yinc2 = 1;
  }
  else{
    /* y is independent variable */
    numpixels = deltay + 1;
    d = (deltax<<1) - deltay;
    dinc1 = deltax<<1;
    dinc2 = (deltax - deltay)<<1;
    xinc1 = 0;
    xinc2 = 1;
    yinc1 = 1;
    yinc2 = 1;
  }
 
  /* Make sure x and y move in the right directions  */
  if (x1 > x2){
    xinc1 = - xinc1;
    xinc2 = - xinc2;
  }
 
  if (y1 > y2){
    yinc1 = - yinc1;
    yinc2 = - yinc2;
  }

  /*  Start drawing at <x1, y1>  */
  x = x1;
  y = y1;

  
  /*  Draw the pixels  */
  for (i = 1; i< numpixels; i++){
	  if (a[x][y]>0){
		  *xout=x; *yout=y;
		  res=1;
		  break;
		  
	  }
		  
    if (d < 0){
      d = d + dinc1;
      x = x + xinc1;
      y = y + yinc1;
    }
    else{
      d = d + dinc2;
      x = x + xinc2;
      y = y + yinc2;
    }
  } /*fin del bucle*/        

  return res;
}
