#include "IntMatSM.h"
#include "Laser.h"
#include "calcul.h"
#include <stdio.h>
#include <math.h>
#include "Bresenham.h"
#include "Scan.h"
#include "Scan.c"

#include "basicOp.h"

#define SECTORES_SM 180

static TSMparams SMparams;


//static float cosAng[361];
//static float sinAng[361];

//float cosAng[361];
//float sinAng[361];



static Tscan scanMax;

static void ComputaMapaSM(float mapa[][DIMMATRIZ],
			  Tsc *sisMat, 
			  Tsc *sisrob,
			  Tscan *scan) ;

static void RayTracingScan(float mapa[][DIMMATRIZ],
   	     Tsc *sisMat, 
		 Tsc *sisrob,
		 Tscan *scan);

static int ObtenerSectorPSM(float p) ;

static void celda2Matriz(int fil, int col, Tpf *out);


void InitScanMatching(){

  short i;
  float ang;

  printf("-- Init Scan ."); 

  InitSM(&SMparams);

  printf(".");

  for (i=0;i<361;i++){
	ang=PI/360*i-PI/2.0;
//    cosAng[i]=cos(ang);
//    sinAng[i]=sin(ang);
	scanMax.laserP[i].r=MAXRANGOLASER;
	scanMax.laserP[i].t=ang;
	pol2car(&scanMax.laserP[i],&scanMax.laserC[i]);	
  }
  scanMax.numPuntos=361;
  
  printf(". OK!\n");  
  
}


int ScanMatching(float mapa[][DIMMATRIZ], Tsc *Smatriz,
		  Tscan *ptos, Tsc *motion, Tsc *Srobot_k1,
		  int flagMapa){

  static int cont = 0;
  static Tscan ptosRef;    
  
  int sLuim;  
  Tsc estimacion;

  if (cont==0){	     
    ptosRef=*ptos;
    sLuim=1;    
  }
  else{       
    if (flagMapa)
		RayTracingScan(mapa,Smatriz, Srobot_k1, &ptosRef);
		//ComputaMapaSM(mapa,Smatriz, Srobot_k1, &ptosRef) ;
    
    
    sLuim=scanMatchingLumilios(&estimacion, 
			       ptos, 
			       &ptosRef,  
			       motion,
			       &SMparams);
    
    if (sLuim)
      *motion=estimacion;            
	ptosRef=*ptos;
  }    

  cont++;
  
  return sLuim;
  
}

static void RayTracingScan(float mapa[][DIMMATRIZ],
   	     Tsc *sisMat, 
		 Tsc *sisrob,
		 Tscan *scan) {
  int i;
  Tsc sisOut,sisRobMat;
  Tpf robMat, outPt;
  Tpcelda robCell, outCell;
  int fil, col;
  
  transfor_inversa_p(sisrob->x,sisrob->y,sisMat,&robMat);
  SisM2celda(&robMat, &robCell);

  inversion_sis(sisrob,&sisOut);
  composicion_sis(&sisOut,sisMat,&sisRobMat);

  scan->numPuntos=0;
  for (i=0; i<NUMPUNTOSLASER; i++) {
	  transfor_inversa_p(scanMax.laserC[i].x,scanMax.laserC[i].y,&sisRobMat,&outPt);
	  SisM2celda(&outPt, &outCell);
	  if (RayTracing(robCell.fil, robCell.col, outCell.fil, outCell.col, mapa, &fil, &col)==1){
		  celda2Matriz(fil,col,&outPt);
		  transfor_directa_p(outPt.x, outPt.y,&sisRobMat,&scan->laserC[scan->numPuntos]);
		  scan->numPuntos++;
	  }
  }
}


static void ComputaMapaSM(float mapa[][DIMMATRIZ],
		 Tsc *sisMat, 
		 Tsc *sisrob,
		 Tscan *scan) {

  int i,j;
  Tsc sisOut,sisRobMat;
  Tpf sol,out;
  float max_valor;
  float dist,ang,d;
  int sec;
  Tpfp sectorizadoP[SECTORES_SM];


  max_valor = 2 * CUADRADO(DIMMATRIZ*DIMCELDA);

  inversion_sis(sisrob,&sisOut);
  composicion_sis(&sisOut,sisMat,&sisRobMat);

  for (i=0; i<SECTORES_SM; i++)
    sectorizadoP[i].r=max_valor;

  for (i=0; i<DIMMATRIZ; i++)
    for (j=0; j<DIMMATRIZ; j++)
      if (mapa[i][j]==LLENO){
	celda2Matriz(i,j,&out);
	transfor_directa_p(out.x,out.y,&sisRobMat,&sol);
	dist=CUADRADO(sol.x)+CUADRADO(sol.y);
	ang=ARCOTANGENTE(sol.x,sol.y);
	sec=ObtenerSectorPSM(ang);
	
	if (sectorizadoP[sec].r>dist){
	  sectorizadoP[sec].r=dist;
	  sectorizadoP[sec].t=ang;
	}
      }

  scan->numPuntos = 0;
  for (i=SECTORES_SM-1; i>-1; i--)
    if (sectorizadoP[i].r!=max_valor){
      d=sqrt(sectorizadoP[i].r);
      scan->laserP[scan->numPuntos].r=d;
      scan->laserP[scan->numPuntos].t=sectorizadoP[i].t;
      scan->laserC[scan->numPuntos].x=d*cos(sectorizadoP[i].t);
      scan->laserC[scan->numPuntos].y=d*sin(sectorizadoP[i].t);
      scan->numPuntos++;
    }
}

static int ObtenerSectorPSM(float p) {

  #define FACTOR (-SECTORES_SM/(2.0F*PI))
  #define SUMANDO ((SECTORES_SM+1.0F)/2.0F)

  return ((int)(FACTOR*p+SUMANDO))%SECTORES_SM;

  #undef SUMANDO
  #undef FACTOR
}

static void celda2Matriz(int fil, int col, Tpf *out){

  out->x=(0.5F+col)*DIMCELDA;
  out->y=(DIMMATRIZ-0.5F-fil)*DIMCELDA;

}
