/***************************************************/
/* Last Revised:
$Id: Matriz.h,v 1.2 2004/02/11 21:12:49 jminguez Exp jminguez $
*/
/***************************************************/

/* 
   Este fichero contiene las definiciones de la matriz y todas
   las funciones de manipulacion de las mismas
*/

#ifndef MatrizFichero
#define MatrizFichero

#include "TData.h"

#define DIMMATRIZ 400  /* numero de celdas de la matriz */
#define DIMCELDA  0.05 /* dimension de una celda en metros */

#define VACIO -0.3
#define LLENO 1.0
#define DESCONOCIDO 0.0

#define RADIO_CENTRAL 1 
/* Area de control de la matriz */ 
/* Si el robot sale fuera la matriz se desplaza para incuirlo dentro */

/**************** ********* ***************/
/**************** DATOS     ***************/
/**************** ********* ***************/

typedef struct {
  int fil;
  int col;
}Tpcelda;

/**************** ********* ***************/
/**************** FUNCIONES ***************/
/**************** ********* ***************/

/* --------------------------------------------------------------------------------------- */  
/* MANIPULACION DE LA MATRIZ                                                               */
/* --------------------------------------------------------------------------------------- */

/* --------------------------------------------------------------------------------------- */
/* Initmatriz                                                                              */ 
/*  .... inicializa la matriz con todas sus celdas a DESCONICIDO                           */ 
/*  .... In: la matriz a iniciar y el sistema del robot actual                             */ 
/*  .... Out: la matriz iniciada, y el sistema de la matriz con el robot en el centro      */

void InitMatriz(float a[][DIMMATRIZ], Tsc *Smatriz, Tsc *Srobot);


/* --------------------------------------------------------------------------------------- */
/* MovimientoMatriz                                                                        */
/* .... desplaza la matriz si el robot sale del centro mas distancia que el RADIO_CENTRAL  */
/* ... In: los sistemas de matriz y robot y la matriz                                      */
/* ... Out: nueva posicion del sistema matriz y la matriz con todas sus celdas cambiadas   */

void MovimientoMatriz(Tsc *Srobot, Tsc *Smatriz,
		      float a[][DIMMATRIZ]);

/* --------------------------------------------------------------------------------------- */
/* MANIPULACION COORDENADAS                                                                */
/* --------------------------------------------------------------------------------------- */

/* --------------------------------------------------------------------------------------- */
/* SisM2celda                                                                              */
/* ... In: Recibe un punto en coordenadas del sistema                                      */
/* ... Out:Devuelve la celda de la matriz en la que esta ese punto                         */

void SisM2celda(Tpf *in,Tpcelda *out);

/* --------------------------------------------------------------------------------------- */
/*  Celda2Sis                                                                              */
/* ... In: Recibe una celda                                                                */
/* ... Out: Devuelve las coordenadas del punto central en coordenadas de la matriz          */

void Celda2M(Tpcelda *in,Tpf *out);

/* --------------------------------------------------------------------------------------- */
/*  CeldaInMatriz                                                                          */
/* ... Comprueba que una celda pertenece a la matriz                                       */
/* ... In: la celda                                                                        */
/* ... Out: return --> 1 esta dentro. return --> 0 esta fuera                              */

int CeldaInMatriz(Tpcelda *in);

/* --------------------------------------------------------------------------------------- */
/*  PuntoMatInMatriz                                                                       */
/* ... Comprueba que un punto en coordenadas de la matriz esta dentro                      */
/* ... In: el punto en coordenadas de la matriz                                            */
/* ... Out: return --> 1 esta dentro. return --> 0 esta fuera                              */

int PuntoMatInMatriz(Tpf *punto);

/* --------------------------------------------------------------------------------------- */
/* RecomputaPuntoFinal                                                                     */
/* ... Compruba que un segmento cae dentro de la matriz, y en caso contrario calcula el    */
/* ... punto de corte de este segmento con la misma.                                       */
/* ... In: pInit y pFinal son los puntos del segmento.                                     */
/* ... Out: pNuevo es el punto de corte (no exactamente el corte sino un DIMCELDA/2 dentro */ 
/* ... Nota: esta funcion asume que el pInit esta ya dentro de la matriz y el pFinal es el */
/* ... que puede caer fuera. Sirve para comprobar que los laserazos caen dentro.           */

int RecomputaPuntoFinal(Tpf *pInit, Tpf *pFinal, Tpf *pNuevo);


#endif 

