---------------------------------------------------------------------
-- Fichero:   computos.adb
-- Autor:     J.L.Villarroel, C.Sagues, J.D.Tardos
-- Version:   v2.2 14-3-2002
-- Proposito: Paquete para mantener ocupado al procesador 
---------------------------------------------------------------------
-- v2.0 27-3-2002, JDT
--   Multiplica los bucles_test por 2 hasta que dura 1 seg
--   La duracion es determinista
-- v2.1 24-4-2002, JDT
--   Corregido el error al calibrar el reloj
-- v2.2 14-3-2002, JDT
--   Pasado a Ada.Real_Time, y anadido mensaje al arrancar
---------------------------------------------------------------------

with Ada.Real_Time, Text_IO;
use  Ada.Real_Time, Text_IO;

package body computos is
  debug: boolean := false;

  package FIO is new Float_IO(float); use FIO ;
  package DIO is new Fixed_IO(duration); use DIO ;
  package LIO is new Integer_IO(long_integer); use LIO ;

  bucles_por_segundo: float ;
  bucles_test: long_integer := 10_000 ;
  dura: duration ;

procedure bucles(n: long_integer) is 
  a: float := 2543.785 ;
  b: float := 365.7326 ;
begin
  for i in 1..n loop
    for j in 1..10 loop
      a:= a/b*a**34 ;
    end loop ;
  end loop ;
end bucles ;

function mide_bucles(n: long_integer) return duration is  
  tini: time ;
  dura: duration;
begin  
  delay 0.001 ;  -- Espera un tick del reloj
  tini:= clock ;
  bucles(bucles_test) ;
  dura:= To_Duration(clock - tini);
  if debug then 
    put("Bucles: "); put(n); 
    put("   Duracion: ");  put(dura) ; put("sg") ; new_line ;
  end if ;
  return dura ;
end mide_bucles ;

procedure computo(tmax: float) is
  -- Realiza calculos inutiles durante aprox. el tiempo indicado 
  n: long_integer ;
  t1, t2: time ;
begin
  n := long_integer(tmax * bucles_por_segundo) ;
  if debug then  
    t1 := clock ; 
  end if ;
  bucles(n) ;
  if debug then 
    t2 := clock ;
    put("   Teor:" ) ; put(tmax,3,3,0); 
    put("   Real:" ) ; put(To_Duration(t2-t1),3,3); new_line ;
  end if ;
end computo ;

begin --computos
  put("Inicializando paquete Computos..."); 
  dura := mide_bucles(bucles_test) ;
  while dura < 0.5 loop  --v2.1
    bucles_test:= 2*bucles_test;
    dura := mide_bucles(bucles_test) ;
  end loop ;
  dura := dura + 0.01;  --v2.1
  bucles_por_segundo:= float(bucles_test) / float(dura) ;
  if debug then
    put("Bucles/segundo: "); put(bucles_por_segundo,7,2,0); new_line;
  end if ;
  put_line("OK");
end computos ;

