 /*
    LMS200Demo: This program demostrates the use of serial communication API
    with LMS-200 manufactured by SICK optics (http://www.sick.de).  
    Copyright (C) 2003.  David Yuen (d.yuen@auckland.ac.nz)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*More information:
While SICK optics has released the serial communication protocol, it does not
work as intended.  (At least for our laser scanner LMS200-30106.  Some packets,
notably most of the ACK packets, have been changed.  Also, by default, they
only distribute a demo program in windows with no source code.  Therefore, I
decided to write another one in C.  The program is supposed to be ANSI C and
POSIX compliant.  Since it is a  demo program, some of the codes are a little
bit bloated.  

The communication protocol can be downloaded from SICK.  Do a search for
TLLMS2xxEN_8007954_04042003.pdf or "LMS telegram listing" on the web.

Although the program is written with LMS-200 in mind, it should work with
similar units like LMS-220, LMS-211, LM-221, possibly with some minor changes
for the declaration of the command and ACK packets.  The electronics of the
laser scanner can support fairly fast updating speed.  The serial link is the
bottle neck (if you don't buy their special 500kbps card).  Therefore, it makes
sense to use the fastest transfer mode (38400 bps) once after power up.  On
the other hand, it should be reset to the default of 9600bps after use.  The
SIGINT, SIGTERM etc are thus trapped to carry out the import scanner transfer
reset process.

In addition to the 500000bps card from SICK a USB to RS422
converter can also be used at 500000bps.  Both the USB-COMi and 
USB-COMi-si have been tried and used with an LMS291.

compile: gcc -Wall -oLMSdemo LMSdemo.c
syntax:  ./LMSdemo [Flags]
Flags:
  -n nrepeat       (number of repeat sets of measurements)
  -r 180|100       (scan range in degree)
  -s 0.25|0.5|1.0  (angular resolution in degree)
  -u cm|mm         (measurement unit)
  -p /dev/tty*    (the com port used)
  -b 9600|19200|38400|500000 (the baud rate to use)
  -h               (help)
The program has been built with both gcc-2.95.4 and gcc-3.2.3 in Debian unstable.

Last update: 2004/07/23
Version: 0.3.0   2004/07/23   
Added CRC checking/ 500k baud support.  Contributed by Kevin 
Knoedler <knoedler AT _no_spam_dslextreme.com>
Version: 0.2.1   2003/03/08
Initial public release from David Yuen <d.yuen AT _no_spam_auckland.ac.nz>
*/

#define _POSIX_SOURCE 1 /* POSIX compliant source */
#define FALSE 0
#define TRUE 1
#define MAXRETRY 25
#define MAXNDATA 802
#define MAXVALORES 401
#define STX 0x02   /*every PC->LMS packet is started by STX*/ 
#define ACKSTX 0x06 /*every PC->LMS packet is started by ACKSTX*/
#define BAUD_500000 512
#define BAUD_38400 256
#define BAUD_19200 128
#define BAUD_9600 64
#define RANGE_100 32
#define RANGE_180 16
#define RES_1_DEG 8
#define RES_0_5_DEG 4
#define RES_0_25_DEG 2 
#define MMMODE 1
#define CMMODE 0
