/*----------------------------------------------------------------------------
 *-------------------------      M a R T E   O S      ------------------------
 *----------------------------------------------------------------------------
 *                                                             V1.59B   070502
 *
 *                                'l a s e r'
 *
 *                                      C
 *
 *  File 'laser.h'                                  by F.J.Feijoo
 *                                            University of Zaragoza (UNIZAR)
 *
 *
 *
 *  ----------------------------------------------------------------------
 *   Copyright (C) 2000-2007, Universidad de Cantabria, SPAIN
 *
 *   MaRTE OS web page: http://marte.unican.es
 *   Contact Addresses: Mario Aldea Rivas          aldeam@unican.es
 *                      Michael Gonzalez Harbour      mgh@unican.es
 *
 *  MaRTE OS  is free software; you can  redistribute it and/or  modify it
 *  under the terms of the GNU General Public License  as published by the
 *  Free Software Foundation;  either  version 2, or (at  your option) any
 *  later version.
 *
 *  MaRTE OS  is distributed  in the  hope  that  it will be   useful, but
 *  WITHOUT  ANY  WARRANTY;     without  even the   implied   warranty  of
 *  MERCHANTABILITY  or  FITNESS FOR A  PARTICULAR PURPOSE.    See the GNU
 *  General Public License for more details.
 *
 *  You should have received  a  copy of  the  GNU General Public  License
 *  distributed with MaRTE  OS;  see file COPYING.   If not,  write to the
 *  Free Software  Foundation,  59 Temple Place  -  Suite 330,  Boston, MA
 *  02111-1307, USA.
 *
 *  As a  special exception, if you  link this  unit  with other  files to
 *  produce an   executable,   this unit  does  not  by  itself cause  the
 *  resulting executable to be covered by the  GNU General Public License.
 *  This exception does  not however invalidate  any other reasons why the
 *  executable file might be covered by the GNU Public License.
 *
 *---------------------------------------------------------------------------*/

#ifndef _LASERDEVICE_H
#define _LASERDEVICE_H

//#define DEBUG 1

#include <drivers/serial_port_driver.h>
#include <pthread.h>
#include <stdint.h>

#include <limits.h>
#define O_NOCTTY 0x0010
#define       SIG_ERR         (void (*)(int))-1
#define  SERIAL_SETATTR              0  /* Set attributes to be
					   written into the UART
					   regs */
#define  SERIAL_GETATTR              1  /* Get attributes from the
					   UART registers */
#define  SERIAL_SETSPEED             2  /* Set input/output speed */
#define  SERIAL_GETSPEED             3  /* Get input/output speed */
#define  SERIAL_ENABLE_INTERRUPTS    4  /* Enable interrupts */
#define  SERIAL_DISABLE_INTERRUPTS   5  /* Disable interrupts */
#define  SERIAL_GET_LAST_LINE_ERROR  6  /* Get error bits in the line
					   status register for the
					   last line status error
					   detected. A pointer to
					   unsigned char is expected
					   in the third argument of
					   ioctl function (arg) */
#define  SERIAL_RESET_LAST_LINE_ERROR  7 /* Reset last line error */
#define  SERIAL_FLUSH                8  /* Flush the buffer */
#define  SERIAL_SET_REL_TIMEOUT      9  /* Set the relative timeout
                                           for blocking read operations */


#include <sys/types.h>
//#include <sys/stat.h>
#include <fcntl.h>
//#include <termios.h>
#include <stdio.h>
#include <sys/time.h>
#include <time.h>
#include <signal.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>

#include <drivers/laser_types.h>
// #define _POSIX_SOURCE 1 /* POSIX compliant source */
// #define FALSE 0
// #define TRUE 1
// #define MAXRETRY 25
// #define MAXNDATA 802
// #define STX 0x02   /*every PC->LMS packet is started by STX*/ 
// #define ACKSTX 0x06 /*every PC->LMS packet is started by ACKSTX*/
// #define BAUD_500000 512
// #define BAUD_38400 256
// #define BAUD_19200 128
// #define BAUD_9600 64
// #define RANGE_100 32
// #define RANGE_180 16
// #define RES_1_DEG 8
// #define RES_0_5_DEG 4
// #define RES_0_25_DEG 2 
// #define MMMODE 1
// #define CMMODE 0
typedef unsigned char uchar;

/*the cmd and ack packets for the 5 different range/resolution modes*/
const uchar PCLMS_RES1[]={0x02,0x00,0x05,0x00,0x3b,0x64,0x00,0x64,0x00};
const uchar PCLMS_RES2[]={0x02,0x00,0x05,0x00,0x3b,0x64,0x00,0x32,0x00};
const uchar PCLMS_RES3[]={0x02,0x00,0x05,0x00,0x3b,0x64,0x00,0x19,0x00};
const uchar PCLMS_RES4[]={0x02,0x00,0x05,0x00,0x3b,0xb4,0x00,0x64,0x00};
const uchar PCLMS_RES5[]={0x02,0x00,0x05,0x00,0x3b,0xb4,0x00,0x32,0x00};
const uchar LMSPC_RES1_ACK[]={0x06,0x02,0x80,0x07,0x00,0xbb,0x01,0x64,0x00,0x64,0x00,0x10};
const uchar LMSPC_RES2_ACK[]={0x06,0x02,0x80,0x07,0x00,0xbb,0x01,0x64,0x00,0x32,0x00,0x10};
const uchar LMSPC_RES3_ACK[]={0x06,0x02,0x80,0x07,0x00,0xbb,0x01,0x64,0x00,0x19,0x00,0x10};
const uchar LMSPC_RES4_ACK[]={0x06,0x02,0x80,0x07,0x00,0xbb,0x01,0xb4,0x00,0x64,0x00,0x10};
const uchar LMSPC_RES5_ACK[]={0x06,0x02,0x80,0x07,0x00,0xbb,0x01,0xb4,0x00,0x32,0x00,0x10};

/*the cmd and ack packets different measurement unit modes*/
const uchar PCLMS_SETMODE[]={0x02,0x00,0x0a,0x00,0x20,0x00,0x53,0x49,0x43,0x4b,0x5f,0x4c,0x4d,0x53};
const uchar PCLMS_MM[]={0x02,0x00,0x23,0x00,0x77,
			 0x00,0x00, // Block A
			 0x46,0x00, // Block B
			 0x01,      // Block C [9]
			 0x00,      // Block D (was 0xd)
			 0x01, // Block E - 0x00 = cm, 0x01 = mm
			 0x00, // Block F
			 0x00, // Block G
			 0x02, // Block H
			 0x02, // Block I
			 0x02, // Block J
			 0x00, // Block K
			 0x00, // Block L
			 0x0a, // Block M
			 0x0a, // Block N
			 0x50, // Block O
			 0x64, // Block P
			 0x00, // Block Q
			 0x0a, // Block R
			 0x0a, // Block S
			 0x50, // Block T
			 0x80, // Block U
			 0x00, // Block V
			 0x0a, // Block W
			 0x0a, // Block X
			 0x50, // Block Y
			 0x64, // Block Z
			 0x00, // Block A1
			 0x00, // Block A2
			 0x00,0x00, // Block A3
			 0x02,0x00};// Block A4
const uchar LMSPC_MM_ACK[]={0x06,0x02,0x80,0x25,0x00,0xf7,0x01,0x00,0x00,0x46,0x00,0x01,0x0d,0x01,\
			    0x00,0x00,0x02,0x02,0x02,0x00,0x00,0x0a,0x0a,0x50,0x64,0x00,0x0a,0x0a,\
			    0x50,0x80,0x00,0x0a,0x0a,0x50,0x64,0x00,0x00,0x00,0x00,0x02,0x00,0x10};

/*the cmd packets for setting transfer speed and  controlling the start and stop of measurement*/
const uchar PCLMS_STATUS[]= {0x02,0x00,0x01,0x00,0x31};
const uchar PCLMS_START[]=  {0x02,0x00,0x02,0x00,0x20,0x24};
const uchar PCLMS_STOP[]=   {0x02,0x00,0x02,0x00,0x20,0x25};
const uchar PCLMS_B9600[] = {0x02,0x00,0x02,0x00,0x20,0x42};
const uchar PCLMS_B19200[]= {0x02,0x00,0x02,0x00,0x20,0x41};
const uchar PCLMS_B38400[]= {0x02,0x00,0x02,0x00,0x20,0x40};
const uchar PCLMS_B500000[]={0x02,0x00,0x02,0x00,0x20,0x48};
/*the ack packet for the above*/
const uchar LMSPC_CMD_ACK[]={0x06,0x02,0x80,0x03,0x00,0xa0,0x00,0x10};

//variable para dejar el puerto serie como estaba
static termios_t OLDTERMIOS;

typedef struct LMS
{
  int LMS_fd;


  //variables globales para añadir funciones estilo player:
  double min_angle, max_angle, scan_res, range_res, scan_count;
  //puntero a un vector de valores doubles que debe definir el usuario
  //double* valores; 
  

  //mutex para bloquear la lectura del laser de forma correcta
  pthread_mutexattr_t mutexattr;
  pthread_mutex_t mutex;

  //vector donde se guardan las medidas del laser
  int valores0[MAXVALORES]; 
  int valores1[MAXVALORES]; 
  int numVector; // tomara el valor 0 o 1 dependiendo del vector que se este escribiendo en ese momento
		// se tendra que leer el vector con indice contrario

  int estado;

} LMS;

//void showdata(int len, uchar *buf);
void showdata(int len, char *buf);

void showLaser(int len, char *buf);

int initLMS(const char *serialdev, int baud_sel);

int setUnits(int unit);

int sendPassword();

int setRangeRes(int res);

void chkstatus(char byte);

int startLMS();

void resetLMS();

void stopLMS();

void connectToLMS(int range_mode, int res_mode, int unit_mode,
			char * port, int baud_sel);

int readLMSdata(uchar* buf);

int readLMSdataDemand(uchar* buf);

int GetCountLaser();

double GetScanResLaser();

double GetRangeResLaser();

int readLMSValues();

int readLMSValuesDemand();

int laserazo (int i);

int getStatus ();

void lockLaser();
void unlockLaser();

#endif


