/*******************************************************
 NAME:		KFPT.h
 AUTHOR:	Luis Montesano
 DATE:		07-04-04
 DESCRIPTION:
	This is the header file for the KFPT functions that 
	implement the tracking of moving objects using Kalman
	Filters. It provides an interface based on the previous
	estimations and the incoming data. It uses the KF_equations
	and the listaKFPT modules.

********************************************************/

#ifndef KFPT_h
#define KFPT_h

#include <stdio.h>
#include "imported/Laser.h"

#include "listaKFPT.h"
#include "listaKFPT.c"

#include "KF_Equations.h"
#include "KF_Equations.c"

#define STATIC 1
#define MOVING 2
#define UNKNOWN 3
#define OUT_OF_RANGE 4

// DATA TYPES
/*typedef struct {
	Tscan outScan;				// Matrix xy coordintes of maximum range scan points
	Tscan inScan;				// Matrix xy coordintes of the valid scan points
	Tscan robotScan;			// Robot xy coordinates of the valid scan points
	Tpcelda scanCells[NUMPUNTOSLASER];  // Matrix cell coordinates of the valid scan points
	Tpcelda outCells[NUMPUNTOSLASER];	// Matrix cell coordinates of maximum range scan points
	short staticInd[NUMPUNTOSLASER];	// Index of static (inScan or robotScan) points
	short movingInd[NUMPUNTOSLASER];	// Index of moving (inScan or robotScan) points
	short unknownInd[NUMPUNTOSLASER];	// Index of unknown (inScan or robotScan) points
	short outInd[NUMPUNTOSLASER];		// Index of out points (this are counted over the original laser data)!!!!
	short tag[NUMPUNTOSLASER];			//
	short staticCnt;					// Number of static points
	short movingCnt; 					// Number of moving points
	short unknownCnt; 					// Number of unknown points
	short outCnt;
} TclassPt;
*/
typedef struct {
	Tscan robotScan;					// Robot xy coordinates of the valid scan points
	short tag[NUMPUNTOSLASER];			// Clasification of the original points
	Tscan staticPtR;					// static points in robot coordinates
	Tscan movingPtR;					// moving points in robot coordinates
	Tscan unknownPtR;					// unknown points in robot coordinates
	Tscan outPtR;						// maximum range points in robot coordinates
	Tscan movingSpuriusPtR;				// moving points in robot coordinates
	Tpcelda scanCells[NUMPUNTOSLASER];
} TclassPt;


// FUNCTIONS
void initKFPT();

void predictFilers(TnodoKFPT *KFS, float period);
void updateFilters(TnodoKFPT **KFS, Tscan const *moving, Tsc *Srobot);

void clasificarPuntos(float *laser, float mapa[][DIMMATRIZ], TnodoKFPT *KFS, Tsc *SRobot, Tsc *SMap, TclassPt *classi);

#endif
