/*
  J. Neira
  J. A. Castellanos
  Robotics and Real Time Group
  University of Zaragoza
  
  sp_matrix.c
  Implements basic MATRIX operations
*/

#include <stdio.h>
#include <stdlib.h>
#include "sp_matrix.h"

/*****************************************************************************/
 MATRIX create_matrix (int rows, int cols)

/*****************************************************************************
 Creates a MATRIX of dimensions (rows, cols) and initializaes it to zeros.
******************************************************************************/
{
  MATRIX m; 

  MROWS (m) = rows;
  MCOLS (m) = cols;

  {
    int i, j;

    for (i = 0; i < MROWS (m); i++)
        for (j = 0; j < MCOLS (m); j++)
            MDATA (m, i, j) = 0;
  }

  return m;
}

/*****************************************************************************/
 void initialize_matrix (MATRIX *m, int rows, int cols)

/*****************************************************************************
 Initializes a MATRIX to dimensions (rows, cols) and content zeros.
******************************************************************************/
{
  MROWS (*m) = rows;
  MCOLS (*m) = cols;

  {
    int i, j;

    for (i = 0; i < MROWS (*m); i++)
        for (j = 0; j < MCOLS (*m); j++)
            MDATA (*m, i, j) = 0;
  }

}

/*****************************************************************************/
 MATRIX CHEPE_diagonal_matrix (int dim, float el1, float el2, float el3)

/*****************************************************************************
 Creates a MATRIX of dimensions (dim, dim) and initializes it to diagonal.
******************************************************************************/
{
  MATRIX m; 

  initialize_matrix (&m, dim, dim);

  MDATA (m, 0, 0) = el1;
  MDATA (m, 1, 1) = el2;
  MDATA (m, 2, 2) = el3;

  return m;
}

/*****************************************************************************/
 void diagonal_matrix (MATRIX *m, int dim, float el1, float el2, float el3)

/*****************************************************************************
 Creates a MATRIX of dimensions (dim, dim) and initializes it to diagonal.
******************************************************************************/
{
  initialize_matrix (m, dim, dim);

  MDATA (*m, 0, 0) = el1;
  MDATA (*m, 1, 1) = el2;
  MDATA (*m, 2, 2) = el3;
}

/*****************************************************************************/
 void print_matrix (char *message, MATRIX const *m)

/*****************************************************************************
 Print to stdout the contents of MATRIX m.
******************************************************************************/
{
  int i, j;

  printf ("%s\n",message);
  printf("%d %d \n",MROWS (*m),MCOLS (*m));
  if ((MROWS (*m) <= MAX_ROWS) && (MCOLS (*m) <= MAX_COLS))
    for (i = 0; i < MROWS (*m); i++)
    {
        for (j = 0; j < MCOLS (*m); j++)
            printf ("%10.5f ", MDATA (*m, i, j));
        printf ("\n");
    }
  else printf ("Dimension incorrecta!");
  printf ("\n");
}

/*****************************************************************************/
 void identity_matrix (MATRIX *m, int d)
 
/*****************************************************************************
 Returns a d x d identity  matrix in m
******************************************************************************/
{
  initialize_matrix (m, d, d);

  {
    int i;

    for (i = 0; i < MROWS (*m); i++)
        MDATA (*m, i, i) = 1;
  }
}

/*****************************************************************************/
 void transpose_matrix (MATRIX const *m, MATRIX *r)

/*****************************************************************************
 Returns the MATRIX transpose of m in r.
******************************************************************************/
{
  int i, j;

  (*r).rows = MCOLS (*m);
  (*r).cols = MROWS (*m);

  for (i = 0; i < MROWS (*m); i++)
      for (j = 0; j < MCOLS (*m); j++)
      MDATA (*r, j, i) = MDATA (*m, i, j);
}


/*****************************************************************************/
 void add_matrix (MATRIX const *m, MATRIX const *n, MATRIX *r)

/*****************************************************************************
 Returns the matricial sum of m and n in r.
******************************************************************************/
{
  if (! (M_EQUAL_DIM (*m, *n)))
  {
     printf ("ERROR (add_matrix): MATRIX dimensions incompatible!\n");
     print_matrix ("MATRIX 1", m);
     print_matrix ("MATRIX 2", n);
     exit(1); /*added 1996-07*/
  }
  else
  {
    int i, j;

    MROWS (*r) = MROWS (*m);
    MCOLS (*r) = MCOLS (*m);

    for (i = 0; i < MROWS (*m); i++)
        for (j = 0; j < MCOLS (*m); j++)
          MDATA (*r, i, j) = MDATA (*m, i, j) + MDATA (*n, i, j);
  }
}

/*****************************************************************************/
 void substract_matrix (MATRIX const *m, MATRIX const *n, MATRIX *r)

/*****************************************************************************
 Returns the matricial substracion of m and n in r.
******************************************************************************/
{
  if (! (M_EQUAL_DIM (*m, *n)))
  {
     printf ("ERROR (substract_matrix): MATRIX dimensions incompatible!\n");
     print_matrix ("MATRIX 1", m);
     print_matrix ("MATRIX 2", n);
     exit(1); /*added 1996-07*/
  }
  else
  {
    int i, j;

    MROWS (*r) = MROWS (*m);
    MCOLS (*r) = MCOLS (*m);

    for (i = 0; i < MROWS (*m); i++)
        for (j = 0; j < MCOLS (*m); j++)
          MDATA (*r, i, j) = MDATA (*m, i, j) - MDATA (*n, i, j);
  }
}

/*****************************************************************************/
 void multiply_matrix (MATRIX const *m, MATRIX const *n, MATRIX *r)

/*****************************************************************************
 Returns the matricial product of m and n in r.
******************************************************************************/
{
  if (! (M_COMPAT_DIM (*m, *n)))
  {
     printf ("ERROR (multiply_matrix): MATRIX dimensions incompatible!\n");
     print_matrix ("MATRIX 1", m);
     print_matrix ("MATRIX 2", n);
     exit(1); /*added 1996-07*/
  }
  else
  {
    int i, j, k;
    float datum;

    MROWS (*r) = MROWS (*m);
    MCOLS (*r) = MCOLS (*n);

    for (i = 0; i < MROWS (*m); i++)
        for (j = 0; j < MCOLS (*n); j++)
        {
          datum = 0;
          for (k = 0; k < MCOLS (*m); k++)
            datum = datum + MDATA (*m, i, k) * MDATA (*n, k, j);
          MDATA (*r, i, j) = datum;
        }
  }
}

/*****************************************************************************/
 void pp_multiply_matrix (MATRIX const *m, MATRIX const *n, MATRIX *r)

/*****************************************************************************
 Returns the matricial product m * n * mt in r.
******************************************************************************/
{
  MATRIX mt, mn;

  transpose_matrix (m, &mt);
  multiply_matrix (m, n, &mn);
  multiply_matrix (&mn, &mt, r);
}

/*****************************************************************************/
 void multiply_matrix_t_matrix(MATRIX const *a, MATRIX const *b, MATRIX *r)
/*****************************************************************************
 Returns the matricial product a * bt in r
 *****************************************************************************/
{
	MATRIX	b_t;
	
	transpose_matrix(b, &b_t);
	multiply_matrix(a, &b_t, r);
}

/*****************************************************************************/
 void multiply_matrix_matrix_t_matrix(MATRIX const *a, MATRIX const *b, 
				      MATRIX const *c, MATRIX *r)
 
/*****************************************************************************
 Returns the matricial product a * b * ct in r
 *****************************************************************************/
{
	MATRIX	ab;
	
	multiply_matrix(a, b, &ab);
	multiply_matrix_t_matrix(&ab, c, r);
}

/*****************************************************************************/
 void multiply_matrix_scalar (float s, MATRIX const *m, MATRIX *r)

/*****************************************************************************
 Returns the scalar-matricial product of s and m in r.
******************************************************************************/
{
  int i, j;

  MROWS (*r) = MROWS (*m);
  MCOLS (*r) = MCOLS (*m);

  for (i = 0; i < MROWS (*m); i++)
      for (j = 0; j < MCOLS (*m); j++)
        MDATA (*r, i, j) = s * MDATA (*m, i, j);
}

/*****************************************************************************/
 void negate_matrix (MATRIX const *m, MATRIX *r)

/*****************************************************************************
******************************************************************************/
{
  int i, j;

  MROWS (*r) = MROWS (*m);
  MCOLS (*r) = MCOLS (*m);

  for (i = 0; i < MROWS (*m); i++)
      for (j = 0; j < MCOLS (*m); j++)
        MDATA (*r, i, j) = -MDATA (*m, i, j);
}

/*****************************************************************************/
 VECTOR create_vector (int elements)

/*****************************************************************************
 Initializes a VECTOR to dimension (elements) and its contents to zeros.
******************************************************************************/
{
  VECTOR v;

  VELEMENTS (v) = elements;

  {
    int i;

    for (i = 0; i < VELEMENTS (v); i++)
        VDATA (v, i) = 0;
  }

  return v;
}

/*****************************************************************************/
 void initialize_vector (VECTOR *v, int elements)

/*****************************************************************************
 Initializes a VECTOR to dimension (elements) and its contents to zeros.
******************************************************************************/
{
  VELEMENTS (*v) = elements;

  {
    int i;

    for (i = 0; i < VELEMENTS (*v); i++)
        VDATA (*v, i) = 0;
  }
}

/*****************************************************************************/
 void print_vector (char *message, VECTOR const *v)
 
/*****************************************************************************
 Print to stdout the contents of VECTOR m.
******************************************************************************/
{
  int i;

  printf ("%s\n",message);
  if (VELEMENTS (*v) <= MAX_ROWS)
    for (i = 0; i < VELEMENTS (*v); i++)
    {
        printf ("%f ", VDATA (*v, i));
        printf ("\n");
    }
  else printf ("Dimension incorrecta!");
  printf ("\n");
}


/*****************************************************************************/
 void add_vector (VECTOR const *v, VECTOR const *w, VECTOR *r)

/*****************************************************************************
 Returns the VECTORial sum of v and w in r.
******************************************************************************/
{
  if (! (V_EQUAL_DIM (*v, *w)))
  {
     printf ("ERROR (add_vector): VECTOR dimensions incompatible!\n");
     print_vector ("VECTOR 1:", v);
     print_vector ("VECTOR 2:", w);
     exit(1); /*added 1996-07*/
  }
  else
  {
    int i;

    VELEMENTS (*r) = VELEMENTS (*v);

    for (i = 0; i < VELEMENTS (*v); i++)
        VDATA (*r, i) = VDATA (*v, i) + VDATA (*w, i);
  }
}

/*****************************************************************************/
 void substract_vector (VECTOR const *v, VECTOR const *w, VECTOR *r)

/*****************************************************************************
 Returns the VECTORial difference of v and w in r.
******************************************************************************/
{
  if (! (V_EQUAL_DIM (*v, *w)))
  {
     printf ("ERROR (substract_vector): VECTOR dimensions incompatible!\n");
     print_vector ("VECTOR 1:", v);
     print_vector ("VECTOR 2:", w);
     exit(1); /*added 1996-07*/
  }
  else
  {
    int i;

    VELEMENTS (*r) = VELEMENTS (*v);

    for (i = 0; i < VELEMENTS (*v); i++)
        VDATA (*r, i) = VDATA (*v, i) - VDATA (*w, i);
  }
}

/*****************************************************************************/
 void multiply_scalar_vector (float s, VECTOR const *v, VECTOR *r)

/*****************************************************************************
 Returns de scalar-vectorial product of s and v in r.
******************************************************************************/
{
  int i;

  VELEMENTS (*r) = VELEMENTS (*v);

  for (i = 0; i < VELEMENTS (*v); i++)
    VDATA (*r, i) = s * VDATA (*v, i);
}

/*****************************************************************************/
 void negate_vector (VECTOR const *v, VECTOR *r)

/*****************************************************************************
******************************************************************************/
{
  int i;

  VELEMENTS (*r) = VELEMENTS (*v);

  for (i = 0; i < VELEMENTS (*v); i++)
    VDATA (*r, i) = -VDATA (*v, i);
}

/*****************************************************************************/
 float cross_product (MATRIX const *m, int f1, int c1, int f2, int c2)

/*****************************************************************************
******************************************************************************/
{
  return MDATA (*m, f1, c1) * MDATA (*m, f2, c2) - MDATA (*m, f1, c2) * MDATA (*m, f2, c1);
}

/*****************************************************************************/
 float determinant (MATRIX const *m)

/*****************************************************************************
******************************************************************************/
{
  if (!M_SQUARE (*m))
  {
     printf ("ERROR (determinant): MATRIX must be square!\n");
     print_matrix ("MATRIX:", m);
     exit(1); /*added 1996-07*/
  }
  else
  {
    float result;

    if (MROWS (*m) == 1)
       result = MDATA (*m, 0, 0);
    else if (MROWS (*m) == 2)
       result = cross_product (m, 0, 0, 1, 1);
    else
       result = MDATA (*m, 0, 0) * cross_product (m, 1, 1, 2, 2)
              - MDATA (*m, 0, 1) * cross_product (m, 1, 0, 2, 2)
              + MDATA (*m, 0, 2) * cross_product (m, 1, 0, 2, 1);

    return result;
  }
}

/*****************************************************************************/
 void inverse_matrix (MATRIX const *m, MATRIX *n)

/*****************************************************************************
******************************************************************************/
{
  if (!M_SQUARE (*m))
  {
     printf ("ERROR (inverse_matrix): MATRIX must be square!\n");
     print_matrix ("MATRIX:", m);
     exit(1); /*added 1996-07*/
  }
  else
  {
    float det;

    det = determinant (m);

    if (det == 0.0)
    {
       printf ("ERROR (inverse_matrix): singular MATRIX!\n");
       print_matrix ("MATRIX:", m);
       exit(1); /*added 1996-07*/
    }
    else
    {
      initialize_matrix (n, MROWS (*m), MCOLS (*m));
      if (MROWS (*m) == 1)
      {
        MDATA (*n, 0, 0) = 1 / det ;
      }
      else if (MROWS (*m) == 2)
      {
        MDATA (*n, 0, 0) = MDATA (*m, 1, 1) / det ;
        MDATA (*n, 0, 1) = -MDATA (*m, 0, 1) / det ;
        MDATA (*n, 1, 0) = -MDATA (*m, 1, 0) / det ;
        MDATA (*n, 1, 1) = MDATA (*m, 0, 0) / det ;
      }
      else
      {
        MDATA (*n, 0, 0) = cross_product (m, 1, 1, 2, 2) / det ;
        MDATA (*n, 0, 1) = -cross_product (m, 0, 1, 2, 2) / det ;
        MDATA (*n, 0, 2) = cross_product (m, 0, 1, 1, 2) / det ;
        MDATA (*n, 1, 0) = -cross_product (m, 1, 0, 2, 2) / det ;
        MDATA (*n, 1, 1) = cross_product (m, 0, 0, 2, 2) / det ;
        MDATA (*n, 1, 2) = -cross_product (m, 0, 0, 1, 2) / det ;
        MDATA (*n, 2, 0) = cross_product (m, 1, 0, 2, 1) / det ;
        MDATA (*n, 2, 1) = -cross_product (m, 0, 0, 2, 1) / det ;
        MDATA (*n, 2, 2) = cross_product (m, 0, 0, 1, 1) / det ;
      }
    }
  }
}

/*****************************************************************************/
 void multiply_matrix_vector (MATRIX const *m, VECTOR const *v, VECTOR *r)

/*****************************************************************************
 Returns the VECTOR-MATRIX product of m and v in r.
******************************************************************************/
{
  if (! (MV_COMPAT_DIM (*m, *v)))
  {
     printf ("ERROR (multiply_matrix_vector): MATRIX  and VECTOR dimensions incompatible!\n");
     print_matrix ("MATRIX:", m);
     print_vector ("VECTOR:", v);
     exit(1); /*added 1996-07*/
  }
  else
  {
    int i, j;
    float datum;

    VELEMENTS (*r) = MROWS (*m);

    for (i = 0; i < MROWS (*m); i++)
    {
        datum = 0;
        for (j = 0; j < VELEMENTS (*v); j++)
            datum = datum + MDATA (*m, i, j) * VDATA (*v, j);
        VDATA (*r, i) = datum;
    }
  }
}

/*****************************************************************************/
 void pp_multiply_t_vector_matrix (VECTOR const *v, MATRIX const *m, float *d)

/*****************************************************************************
******************************************************************************/
{
  VECTOR w;
  int i;

  multiply_matrix_vector (m, v, &w);

  *d = 0;
  for ( i = 0; i < VELEMENTS (*v); i++)
      *d += VDATA (*v, i) * VDATA (w, i);
}

/*****************************************************************************/
 BMAT bmat (int first, int second, int third)

/*****************************************************************************
******************************************************************************/
{
  BMAT b;

  FIRST (b) = first;
  SECOND (b) = second;
  THIRD (b) = third;
  RANGE (b) = first+second+third;
  
  return b;
}

/*****************************************************************************/
 void initialize_bmat (BMAT *b, int first, int second, int third)

/*****************************************************************************
******************************************************************************/
{
 FIRST (*b) = first;
 SECOND (*b) = second;
 THIRD (*b) = third;
 RANGE (*b) = first+second+third;
}

/*****************************************************************************/
 void print_bmat (char *message, BMAT const *b)

/*****************************************************************************
******************************************************************************/
{
 printf ("%s (%d, %d, %d)\n\n", message, FIRST (*b), SECOND (*b), THIRD (*b));
}

/*****************************************************************************/
 void multiply_bmat_vector (BMAT const *b, VECTOR const *v, VECTOR *r)

/*****************************************************************************
******************************************************************************/
{
  VELEMENTS (*r) = RANGE (*b);
  if (FIRST (*b))
     VDATA (*r, 0) = VDATA (*v, 0);
  if (SECOND (*b))
     VDATA (*r, FIRST (*b)) = VDATA (*v, 1);
  if (THIRD (*b))
     VDATA (*r, FIRST (*b) + SECOND (*b)) = VDATA (*v, 2);
}

/*****************************************************************************/
 void multiply_t_bmat_vector (BMAT const *b, VECTOR const *v, VECTOR *r)

/*****************************************************************************
******************************************************************************/
{
  VELEMENTS (*r) = DOF;
  VDATA (*r, 0) = (FIRST (*b))  ? VDATA (*v, 0) : 0;
  VDATA (*r, 1) = (SECOND (*b)) ? VDATA (*v, FIRST (*b)) : 0;
  VDATA (*r, 2) = (THIRD (*b))  ? VDATA (*v, FIRST (*b) + SECOND (*b)) : 0;
}

/*****************************************************************************/
 void copy_row (MATRIX const *s, MATRIX *d, int srow, int drow, int cols)

/*****************************************************************************
******************************************************************************/
{
  int i;

  for (i = 0; i < cols; i++)
      MDATA (*d, drow, i) = MDATA (*s , srow, i);
}

/*****************************************************************************/
 void copy_col (MATRIX const *s, MATRIX *d, int scol, int dcol, int rows)

/*****************************************************************************
******************************************************************************/
{
  int i;

  for (i = 0; i < rows; i++)
      MDATA (*d, i, dcol) = MDATA (*s , i, scol);
}

/*****************************************************************************/
 void pp_multiply_bmat_matrix (BMAT const *b, MATRIX const *m, MATRIX *r)

/*****************************************************************************
******************************************************************************/
{
  MATRIX n;

  MROWS (n) = RANGE (*b);
  MCOLS (n) = MCOLS (*m);
  if (FIRST (*b))
     copy_row (m, &n, 0, 0, MCOLS (*m));
  if (SECOND (*b))
     copy_row (m, &n, 1, FIRST (*b), MCOLS (*m));
  if (THIRD (*b))
     copy_row (m, &n, 2, FIRST (*b) + SECOND (*b), MCOLS (*m));

  MROWS (*r) = RANGE (*b);
  MCOLS (*r) = RANGE (*b);
  if (FIRST (*b))
     copy_col (&n, r, 0, 0, MROWS (n));
  if (SECOND (*b))
     copy_col (&n, r, 1, FIRST (*b), MROWS (n));
  if (THIRD (*b))
     copy_col (&n, r, 2, FIRST (*b) + SECOND (*b), MROWS (n));
}

/*****************************************************************************/
 void fill_row (MATRIX *d, int row, int cols)

/*****************************************************************************
******************************************************************************/
{
  int i;

  for (i = 0; i < cols; i++)
      MDATA (*d, row, i) = 0;
}

/*****************************************************************************/
 void fill_col (MATRIX *d, int col, int rows)

/*****************************************************************************
******************************************************************************/
{
  int i;

  for (i = 0; i < rows; i++)
      MDATA (*d, i, col) = 0;
}

/*****************************************************************************/
 void pp_multiply_t_bmat_matrix (BMAT const *b, MATRIX const *m, MATRIX *r)

/*****************************************************************************
******************************************************************************/
{
  MATRIX n;

  MROWS (n) = DOF;
  MCOLS (n) = MCOLS (*m);

  if (FIRST (*b))
     copy_row (m, &n, 0, 0, MCOLS (*m));
  else 
     fill_row (&n, 0, MCOLS (*m));

  if (SECOND (*b))
     copy_row (m, &n, FIRST (*b), 1, MCOLS (*m));
  else 
     fill_row (&n, 1, MCOLS (*m));

  if (THIRD (*b))
     copy_row (m, &n, FIRST (*b) + SECOND (*b), 2, MCOLS (*m));
  else 
     fill_row (&n, 2, MCOLS (*m));

  MROWS (*r) = DOF;
  MCOLS (*r) = DOF;

  if (FIRST (*b))
     copy_col (&n, r, 0, 0, MROWS (n));
  else 
     fill_col (r, 0, MROWS (n));

  if (SECOND (*b))
     copy_col (&n, r, FIRST (*b), 1, MROWS (n));
  else 
     fill_col (r, 1, MROWS (n));

  if (THIRD (*b))
     copy_col (&n, r, FIRST (*b) + SECOND (*b), 2, MROWS (n));
  else 
     fill_col (r, 2, MROWS (n));

}

/*****************************************************************************/
 void multiply_bmat_matrix (BMAT const *b, MATRIX const *m, MATRIX *r)

/*****************************************************************************
******************************************************************************/
{
  MROWS (*r) = RANGE (*b);
  MCOLS (*r) = MCOLS (*m);
  if (FIRST (*b))
     copy_row (m, r, 0, 0, MCOLS (*m));
  if (SECOND (*b))
     copy_row (m, r, 1, FIRST (*b), MCOLS (*m));
  if (THIRD (*b))
     copy_row (m, r, 2, FIRST (*b) + SECOND (*b), MCOLS (*m));
}

/*****************************************************************************/
 void multiply_matrix_t_bmat (MATRIX const *m, BMAT const *b, MATRIX *r)

/*****************************************************************************
******************************************************************************/
{
  MROWS (*r) = MROWS (*m);
  MCOLS (*r) = RANGE (*b);
  if (FIRST (*b))
     copy_col (m, r, 0, 0, MROWS (*m));
  if (SECOND (*b))
     copy_col (m, r, 1, FIRST (*b), MROWS (*m));
  if (THIRD (*b))
     copy_col (m, r, 2, FIRST (*b) + SECOND (*b), MROWS (*m));
}

/*****************************************************************************/
 void multiply_matrix_bmat (MATRIX const *m, BMAT const *b, MATRIX *r)

/*****************************************************************************
******************************************************************************/
{
  MROWS (*r) = MROWS (*m);
  MCOLS (*r) = 3;
  copy_col (m, r, 0, 1, MROWS (*m));
  copy_col (m, r, 1, 2, MROWS (*m));
}

