/*------------------------------------------------------------------------
 *---------------------           RT-WMP     		  --------------------
 *------------------------------------------------------------------------
 *                                                         V3.0B  25/07/07
 *
 *       					  'l l _ c o m  _ h'
 *
 *                                      
 *
 *  File 'll_com.h'       								by Danilo Tardioli
 *
 *  ----------------------------------------------------------------------
 *  Copyright (C) 2000-2007, Universidad de Zaragoza, SPAIN
 *
 *  Contact Addresses: Danilo Tardioli	        		 dantard@unizar.es
 *
 *  RT-WMP is free software; you can  redistribute it and/or  modify it
 *  under the terms of the GNU General Public License  as published by the
 *  Free Software Foundation;  either  version 2, or (at  your option) any
 *  later version.
 *
 *  RT-WMP  is distributed  in the  hope  that  it will be   useful, but
 *  WITHOUT  ANY  WARRANTY;     without  even the   implied   warranty  of
 *  MERCHANTABILITY  or  FITNESS FOR A  PARTICULAR PURPOSE.    See the GNU
 *  General Public License for more details.
 *
 *  You should have received  a  copy of  the  GNU General Public  License
 *  distributed with MaRTE  OS;  see file COPYING.   If not,  write to the
 *  Free Software  Foundation,  59 Temple Place  -  Suite 330,  Boston, MA
 *  02111-1307, USA.
 *
 *  As a  special exception, if you  link this  unit  with other  files to
 *  produce an   executable,   this unit  does  not  by  itself cause  the
 *  resulting executable to be covered by the  GNU General Public License.
 *  This exception does  not however invalidate  any other reasons why the
 *  executable file might be covered by the GNU Public License.
 *
 *---------------------------------------------------------------------------*/

#include <sys/types.h>
#include <sys/time.h>  
#include <sys/socket.h>
#include <arpa/inet.h>
#include <unistd.h>
 
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "errno.h"
#include <pthread.h>
#include "core/include/definitions.h"
#include "core/interface/wmp_protected_interface.h"
#include "core/interface/wmp_interface.h"
#include "core/include/wmp_misc.h"
int rx,tx; /* TX and RX sockets*/
char IPADDR[20],IPBROADCAST[20];
char param[20],val[20];
int sim=0;
void* bridge(void *);

int readllcfg(){
	FILE *f;
	f=fopen(".//rt-wmp.ll","r");
	if (f==0) {
		wmprintf(2,"File rt-wmp.ll not found.\n");
		return -1;
	}	
	wmprintf(2,"Reading Low Level Configuration file..."); 
	while (fscanf(f,"%s %s",param,val)!=EOF){ 
		if (strcmp(param,"IP_ADDRESS")==0){
			strcpy(IPADDR,val);
		}
		if (strcmp(param,"BROADCAST_ADDRESS")==0){
			strcpy(IPBROADCAST,val);
		}
		if (strcmp(param,"SIMULATION")==0){
			sim=atoi(val);
		}
	}
	wmprintf(2,"Done\n");
	return 0;
}

void closeLowLevelCom(){
}
int initLowLevelCom(){
	struct sockaddr_in rx_addr,tx_addr;
	int ret=0;
	int txport=4003,rxport=4003;
	if (readllcfg()==-1) ret--;
	if (sim){
		fprintf(stderr,"Modify param\n");
		txport=4103;
		rxport=4003+wmpGetNodeId();
		strcpy(IPADDR,"localhost");
		strcpy(IPBROADCAST,"localhost");
				
	}
	memset(&rx_addr,0,sizeof(rx_addr));
    rx_addr.sin_family=PF_INET;
    rx_addr.sin_port=htons(rxport);
    rx_addr.sin_addr.s_addr=inet_addr(IPADDR);
	
	memset(&tx_addr,0,sizeof(tx_addr));
	tx_addr.sin_family=PF_INET;
    tx_addr.sin_port=htons(txport);
    tx_addr.sin_addr.s_addr=inet_addr(IPBROADCAST);
	
	
	if ((rx=socket(PF_INET,SOCK_DGRAM,IPPROTO_UDP)) <0){
		wmprintf(2,"Error creating rx socket.\n");
		ret--;
	}
	if (bind(rx, (struct sockaddr*)&rx_addr,sizeof(struct sockaddr_in))<0){
		wmprintf(2,"RX BIND ERROR: %s.\n",inet_ntoa(rx_addr.sin_addr));
		ret--;
	}
	if ((tx=socket(PF_INET,SOCK_DGRAM,IPPROTO_UDP)) <0){
		wmprintf(2,"Error creating tx socket.\n");
		ret--;
	}
	int val=1;
	setsockopt(tx,SOL_SOCKET,SO_BROADCAST,&val,sizeof(val));
	if(connect(tx, (struct sockaddr*)&tx_addr,sizeof(struct sockaddr_in))<0){
		wmprintf(2,"Connect error:%s.\n",inet_ntoa(tx_addr.sin_addr));
		ret--;
	}
	if (sim!=0 && wmpGetNodeId()==0){
		pthread_t tid;
		pthread_create(&tid,NULL,bridge,NULL);
	}
	if (ret==0) return 1;
	else return 0;
}

int llsend(char * f,int size){
	int nbytes=send(tx,f,size,0);
	recvfrom(rx,0,0,0,0,0);
	return nbytes;
}

int llreceive(char *f,int timeout){
	int r=0; struct timeval tv;
	if (timeout>0){
		fd_set fd_rx;
		
		tv.tv_sec=0;
		tv.tv_usec=1000*timeout; /* timeout in ms and not us */	
		FD_ZERO(&fd_rx);
		FD_SET(rx, &fd_rx); 
		/* wait TIMEOUT for acknoledwge frame or for net inactivity*/
		r=select(FD_SETSIZE,&fd_rx,NULL,NULL,&tv); 
		if (r){ /* if something is present read the frame and return the frame */			
			recvfrom(rx,(char *)f,MTU,0,0,0);			
			return RECEIVE_OK;
		}else{			
			return EXPIRED;
		}
	}else{
		recvfrom(rx,(char *)f,MTU,0,0,0);
		return RECEIVE_OK;
	}
}

void* bridge(void * param){
	int rx,tx,i;
	fprintf(stderr,"Starting WMP BRIDGE...");
	struct sockaddr_in rx_addr,tx_addr;
	memset(&rx_addr,0,sizeof(rx_addr));
    rx_addr.sin_family=PF_INET;
    rx_addr.sin_port=htons(4103);
    rx_addr.sin_addr.s_addr=inet_addr("localhost");
    
    memset(&tx_addr,0,sizeof(tx_addr));
	tx_addr.sin_family=PF_INET;    
    tx_addr.sin_addr.s_addr=inet_addr("localhost");	
    int ret=0;
    if ((rx=socket(PF_INET,SOCK_DGRAM,IPPROTO_UDP)) <0){
		wmprintf(2,"Error creating BRIDGE rx socket.\n");
		ret--;
	}
	if (bind(rx, (struct sockaddr*)&rx_addr,sizeof(struct sockaddr_in))<0){
		wmprintf(2,"RX BIND ERROR in BRIDGE: %s.\n",inet_ntoa(rx_addr.sin_addr));
		ret--;
	}
	if ((tx=socket(PF_INET,SOCK_DGRAM,IPPROTO_UDP)) <0){
		wmprintf(2,"Error creating BRDIGE tx socket.\n");
		ret--;
	}
	int val=1;
	setsockopt(tx,SOL_SOCKET,SO_BROADCAST,&val,sizeof(val));
	fprintf(stderr,"Done.\n");
	/* BRIDGE */
	char f[2500];	
	while (1){
		usleep(10000);
		int nbytes=recvfrom(rx,(char *)f,MTU,0,0,0);					
		for (i=0;i<wmpGetNumOfNodes();i++){
		  	tx_addr.sin_port=htons(4003+i);
		  	sendto(tx,f,nbytes,0,(struct sockaddr *) &tx_addr,sizeof(tx_addr));		  			  			  				 
		}
		/* FOR the visualizer */
		tx_addr.sin_port=htons(4033);
		sendto(tx,f,nbytes,0,(struct sockaddr *) &tx_addr,sizeof(tx_addr));				
	}
}

