#ifndef QUEUE_C_H_
#define QUEUE_C_H_

#include "config/compiler.h"
#include <string.h>
#include <stdio.h>
#include "core/include/wmp_misc.h"
#include "core/interface/Msg.h"

#define FIFO 		0
#define LIFO 		1
#define UNSORTED 	2

#define REPLACE 	0
#define DROP		1

typedef struct {
		Msg m;
		unsigned long ts;
		int valid;  
}TimeStampedMsg;

typedef struct {
	TimeStampedMsg* tsmsg;
	int rRule;
	int sRule;
	int queueSize; 
	TimeStampedMsg lastPopped;
	SEM_T mutex;
	SEM_T pauseMutex;
	int queueIdx;
	char name[20];
}Queue_t;

void initQueue(Queue_t *q, int size,int sRule,int rRule, char * name);
void freeQueue(Queue_t *q);
void setReplaceRule(Queue_t *q,int rRule);
void setSortRule(Queue_t *q,int sRule);
int pushMsg(Queue_t *q, Msg * p);
int pushMsgWithTS(Queue_t *q, Msg * m, unsigned long ts);
void rescheduleLast(Queue_t *q);
void getLast(Queue_t *q, Msg* p);
long popMsg(Queue_t *q,Msg* p);
char getHighestPriority(Queue_t *q);
unsigned long getNumOfElemInQueue(Queue_t *q);
unsigned long getFreePositions(Queue_t *q);
int getHighestPrioritysAge(Queue_t *q);


#endif /*QUEUE_C_H_*/
