/*------------------------------------------------------------------------
 *---------------------           RT-WMP     		  --------------------
 *------------------------------------------------------------------------
 *                                                         V3.0B  25/07/07
 *
 *                   		  'd i j k s t r a _ c'
 *
 *                                      
 *
 *  File 'dijkstra.c'                     				by Danilo Tardioli
 *
 *  ----------------------------------------------------------------------
 *  Copyright (C) 2000-2007, Universidad de Zaragoza, SPAIN
 *
 *  Contact Addresses: Danilo Tardioli	        		 dantard@unizar.es
 *
 *  RT-WMP is free software; you can  redistribute it and/or  modify it
 *  under the terms of the GNU General Public License  as published by the
 *  Free Software Foundation;  either  version 2, or (at  your option) any
 *  later version.
 *
 *  RT-WMP  is distributed  in the  hope  that  it will be   useful, but
 *  WITHOUT  ANY  WARRANTY;     without  even the   implied   warranty  of
 *  MERCHANTABILITY  or  FITNESS FOR A  PARTICULAR PURPOSE.    See the GNU
 *  General Public License for more details.
 *
 *  You should have received  a  copy of  the  GNU General Public  License
 *  distributed with RT-WMP;  see file COPYING.   If not,  write to the
 *  Free Software  Foundation,  59 Temple Place  -  Suite 330,  Boston, MA
 *  02111-1307, USA.
 *
 *  As a  special exception, if you  link this  unit  with other  files to
 *  produce an   executable,   this unit  does  not  by  itself cause  the
 *  resulting executable to be covered by the  GNU General Public License.
 *  This exception does  not however invalidate  any other reasons why the
 *  executable file might be covered by the GNU Public License.
 *
 *---------------------------------------------------------------------------*/
 
#include "include/dijkstra.h"

static char **m;
static int *marcas;
static int *w; /* w the weight of any step*/
static int *dij;/*dij contains the predecessor of any node*/
static int *tmp;
static int N_NODES;

void initDijkstra(int numNodes){
	int i;
	N_NODES=numNodes;
	m= (char**) MALLOC(N_NODES*sizeof(char*));
	for (i=0;i<N_NODES;i++){
		m[i]=(char *) MALLOC(N_NODES);
	}
	marcas=(int *) MALLOC(N_NODES*sizeof(int));
	w=(int *) MALLOC(N_NODES*sizeof(int));
	dij=(int *) MALLOC(N_NODES*sizeof(int));
	tmp=(int *) MALLOC(N_NODES*sizeof(int));
}

char f_lqm(char val){
	/* val is a % value (0-100) */	
	if (val<25) return 100;
	else if (val>=25 && val <40) return 3;
	else if (val>=40 && val <55) return 2;
	else return 1;
}

int nextStepWithCost(char **lqm,int src,int dst, int* cost){
	int i,j;
	for (i=0;i<N_NODES;i++){
		for (j=0;j<N_NODES;j++){		
			if (lqm[i][j]==0 || i==j) {
				m[i][j]=-1; /*prepare matrix for dijkstra algorithm (absence of link)*/
			}else{ /*snr to LQI conversion, plain at the moment*/
				m[i][j]=f_lqm(lqm[i][j]);		
			}
		}
	}
	int p[N_NODES]; 		
	int len=steps(m,src,dst,p,cost);
	if (len >0) return p[1];
	else return -1;	
}

int isIsolated(char **lqm,int src,int dst){
	int cost;
	int ret=nextStepWithCost(lqm,src,dst,&cost);
	return (ret==-1);
}
 
int nextVertex(int *marcas,int *weight){
	int i, light = -1;
	for (i=0; i<N_NODES; i++){
		if (!marcas[i] && weight[i] != -1){
			if (light == -1 || weight[light] > weight[i]){
				light = i;
			}
		}
	}
	return light;
}

void dijkstra(char **lqm,int v,int * prec,int *weight){

	int c, i;
	memset(marcas,0,N_NODES*sizeof(int));
	for (i=0; i<N_NODES; i++){
		weight[i] = lqm[v][i]; //weights(lq,v, i,N);
		if (weight[i]!=-1) prec[i]=v;
		else prec[i]=-1;
		prec[v]=v; // predecessor of start vertex is himself
	}
	marcas[v] = 1;
	weight[v] = 0;
	while ((v = nextVertex(marcas, weight)) != -1){
		marcas[v]=1;
		for (i=0; i<N_NODES; i++){
			if (!marcas[i] && (c=lqm[v][i]) != -1){
				if (weight[i] == -1){ // non ancora raggiunto
					weight[i] = weight[v] + c;
					prec[i]=v;
				} else 
				if (weight[i]>weight[v]+c){
					weight[i]=weight[v]+c; //raggiunto prima 
					prec[i]=v;
				}
			}
		}
	}
}

/* This function set the path variable to the entire path from src to dst with these two nodes too
the len return value say the result len e.j. if src=4 and dst=9, the result is array path=4 5 7 3 9 len=4 (number of hops).
The lqm MUST be an adjacent matrix with non-existent link value of -1*/

int steps(char **lqm,int src,int dst,int *path, int * cost){

	int len;
	dijkstra(lqm,src,dij,w);
	int v=dst,idx=0;
	if (dij[v]!=-1){
		int tmp[N_NODES];
		while (dij[v]!=src){		
			v=dij[v];
			tmp[idx]=v;
			idx++;
		}	
		tmp[idx]=src;
		int i=0;
		for (i=0;i<=idx;i++){
			path[i]=tmp[idx-i];
		}
		path[i]=dst;
		*cost=w[dst];
		len=idx+1;
	}else{ /* No PATH to destination*/
		*cost=-1;
		len=-1;
	}
	return len;	
}


