/*------------------------------------------------------------------------
 *---------------------           RT-WMP     		  --------------------
 *------------------------------------------------------------------------
 *                                                         V3.0B  25/07/07
 *
 *                   		  'm a n a g e _ c'
 *
 *                                      
 *
 *  File 'manage.c'                     				by Danilo Tardioli
 *
 *  ----------------------------------------------------------------------
 *  Copyright (C) 2000-2007, Universidad de Zaragoza, SPAIN
 *
 *  Contact Addresses: Danilo Tardioli	        		 dantard@unizar.es
 *
 *  RT-WMP is free software; you can  redistribute it and/or  modify it
 *  under the terms of the GNU General Public License  as published by the
 *  Free Software Foundation;  either  version 2, or (at  your option) any
 *  later version.
 *
 *  RT-WMP  is distributed  in the  hope  that  it will be   useful, but
 *  WITHOUT  ANY  WARRANTY;     without  even the   implied   warranty  of
 *  MERCHANTABILITY  or  FITNESS FOR A  PARTICULAR PURPOSE.    See the GNU
 *  General Public License for more details.
 *
 *  You should have received  a  copy of  the  GNU General Public  License
 *  distributed with RT-WMP;  see file COPYING.   If not,  write to the
 *  Free Software  Foundation,  59 Temple Place  -  Suite 330,  Boston, MA
 *  02111-1307, USA.
 *
 *  As a  special exception, if you  link this  unit  with other  files to
 *  produce an   executable,   this unit  does  not  by  itself cause  the
 *  resulting executable to be covered by the  GNU General Public License.
 *  This exception does  not however invalidate  any other reasons why the
 *  executable file might be covered by the GNU Public License.
 *
 *---------------------------------------------------------------------------*/
 
#include "include/manage.h"

/*localLQM[i][j]=> node i hear j with quality localLQM[i][j] */
/* node j is heard with quality localLQM[i][j] from node i*/

int createNewToken(wmpFrame * t){
	EVENT_NEW_TOKEN(t);
	int i=0,l=0;
	t->hdr.serial=status.highestSerial;
	t->hdr.to=UNDEF;
	t->hdr.type=TOKEN;
	t->hdr.retries=0;
	t->tkn.maxPri=UNDEF;
	t->tkn.idMaxPri=UNDEF;
	t->tkn.acknow=status.acknow;
	//t->tkn.nt=1;
	for (i=0;i<status.N_NODES;i++) nstat_clearReached(i); //status.reached[i]=0; /*clear reached field*/
	nstat_setReached(status.id);
	nstat_clearLost(status.id);
	for (i=0;i<status.N_NODES;i++){ /* Exclude isolated node */ 		
		if (isIsolated(lqm_get_ptr(),status.id,i)){
			nstat_setLost(i); //status.reached[i]=2; //ISOLATED						
			int lastTry=nstat_get_val(i);  //status.localLQM[i][i];
			for (l=0;l<status.N_NODES;l++){
				lastTry++; 
				if (lastTry>=status.N_NODES) lastTry=0;
				if (!isIsolated(lqm_get_ptr(),status.id,lastTry)){
					break;
				}
			}
			nstat_set_val(i,lastTry);
		}
	}
	return	EVALUATE_TOKEN;
}

int evaluate_token(wmpFrame * t){
	int i,j;
	/* I have been reached */
	nstat_setReached(status.id);//status.reached[status.id]=1;
	nstat_clearLost(status.id);
	/* Update the age of the message and the FAKE LQM*/
	if (t->hdr.from!=status.id){
		t->tkn.age+=status.perHopDelay;
		if (status.haveFakeLQM){
			for ( i=0;i<status.N_NODES;i++){
				for ( j=0;j<status.N_NODES;j++){
					lqm_set_val(i,j,status.fakeLQM[i][j]);
					//status.localLQM[i][j]=status.fakeLQM[i][j];
				}
			} 
		}
	}
	
	/* I'll send the frame (N.B. Used to see if I'm re-evaluating a frame or not)*/
	t->hdr.from=status.id;
	if (status.acknow!=-1){
		if (t->tkn.acknow!=status.acknow){
			//reschedule();						
		}
		status.acknow=-1;
	}
	
	EVENT_TOKEN_RECEIVED(t);
	
	/*Priority Control*/	
	char highestPriority=getHighestPriorityInTxUCQueue();
	int age=  getHighestPriorityAgeInTxUCQueue();
	
	if (highestPriority!=UNDEF){
		if (highestPriority > t->tkn.maxPri){//control if I have a message more prioritary than the others*/
			t->tkn.idMaxPri=status.id;
			t->tkn.maxPri=highestPriority;
			t->tkn.age=age;
		}else if (highestPriority == t->tkn.maxPri){
			if (t->tkn.age<age){
				t->tkn.idMaxPri=status.id;
				t->tkn.maxPri=highestPriority; //not necessary, the value is the same
				t->tkn.age=age;
			}		
		}
	}
	/*control if I have to search someone*/
	for ( i=0;i<status.N_NODES;i++){
		if (nstat_isLost(i) && !nstat_isReached(i)){// (status.reached[i]==2){
			if (nstat_get_val(i)==status.id){/* Fuck! I've to search someone!*/ //status.localLQM[i][i]==status.id){
				//wmprintf(BOTTOM,"Searching node %d (serial:%d)...\n",i,t->hdr.serial);
				nstat_setReached(i); //status.reached[i]=3; /*I've searched it*/
				t->hdr.to=i;
				return SEND;
			}
		}
	}
	/* Control if I can see someone */
	int tot=0;
	for ( i=0;i<status.N_NODES;i++){
		if (i==status.id) continue;
		tot+=lqm_get_val(status.id,i);//status.localLQM[status.id][i];
	}
	
	if (tot==0){ /* I have the token but cannot see nothing -> i will search all */
		for ( i=0;i<status.N_NODES;i++){
			if (i==status.id) {
				nstat_clearLost(status.id);
				continue;
			}
			nstat_setLost(i);//status.reached[i]=2;
			nstat_clearReached(i);//status.reached[i]=2; 			
			nstat_set_val(i,status.id);//status.localLQM[i][i]=status.id;
		}
		return EVALUATE_TOKEN;
	}		
	/*control if I'm the last one*/
	int res=1;
	for ( i=0;i<status.N_NODES;i++){
		if (i==status.id) continue;
		res=res && nstat_isReached(i);//status.reached[i]; /* with && is ok 2, 3 and 4 values (tosearch and searched)*/
	}	
	if (res>0){ /* I'm the last one */		
		if (t->tkn.idMaxPri>=0){ /* if there is a message to transmit */
			
			EVENT_PREPARING_AUTH(t);
			
			t->aut.dest=t->tkn.idMaxPri;
			t->aut.src=status.id;			
			t->hdr.from=status.id;
			t->hdr.type=AUTHORIZATION;
			t->hdr.retries=0;
			if (t->aut.dest==status.id) { //is an authorization for me
				return EVALUATE_AUTHORIZATION;
			}else{
				int cost;
				int ns=nextStepWithCost(lqm_get_ptr(),status.id,t->tkn.idMaxPri,&cost); /*t->lqm*/
				if (ns>=0) {
					t->hdr.to=ns;				
					return SEND;
				}
				else return NEW_TOKEN;
			}
		}else{ /* Noone have to transmit nothing - start a new trip */
			return NEW_TOKEN;			
		}
	}else {/* I'm not the last one */				
		/*Chooses bestRSSIid between not yet reached nodes*/

/*		ADDING ALEATORITY
 
 		int bestRssi=0;
		int bestRssiId=UNDEF;
		for ( i=0;i<status.N_NODES;i++){
			if (i==status.id) continue;
			if (!nstat_isReached(i)){//status.reached[i]==0){
				if (lqm_get_val(status.id,i)>bestRssi){
					bestRssi=lqm_get_val(status.id,i);
					bestRssiId=i;
				} 
			}
		}
*/	
		int bestRssi=40, idx=0;
		int bestRssiId=UNDEF;
		char ok[status.N_NODES];
		for ( i=0;i<status.N_NODES;i++){
			if (i==status.id) continue;
			if (!nstat_isReached(i)){//status.reached[i]==0){
				if (lqm_get_val(status.id,i)>bestRssi){					
					ok[idx]=i;
					idx++;
				} 
			}
		}
		if (idx>0){
			int v=rand()%idx;
			bestRssiId=ok[v];
		}
		if (bestRssiId==UNDEF){// I cant ear anyone that has not been still reached, go back (N.B. I'm NOT the last one)
			int next_step=UNDEF;
			int minCost=10000;
			for ( i=0;i<status.N_NODES;i++){ //calculate the best path to a not yet reached node with the lowest cost 
				if (!nstat_isLost(i) && !nstat_isReached(i)){//status.reached[i]==0){
					int cost; 
					int ns=nextStepWithCost(lqm_get_ptr(),status.id,i,&cost); /*t->lqm*/
					if (ns!=UNDEF){ /*No path!*/
						if (cost<minCost){
							minCost=cost;
							next_step=ns;
					 	} 
					}
				}
			}
			if (next_step==UNDEF){/* Either I'm Isolated with token or the chosen node is isolated (should be impossible!)*/				
				//wmprintf(2,"CHOSEN NODE ISOLATED -> NEW TOKEN\n");
				return NEW_TOKEN;
			}else{
				t->hdr.to=next_step;		
				return SEND; 
			}
			/* Execution cannot reach this point */
			ASSERT(0);
		}
		else{// I can ear someone that has not been still reached, go forward
			t->hdr.to=bestRssiId;
			return SEND;			
		}
	}	
}


int manage_token_expired_timeout(wmpFrame* t){/* token timeout expired*/
/* here we have 2 types of politics:
	1. retry transmission
	2. set not responding node to reached ad go on
  we can retry (1) up to reach 2n-3 hops in total but the risk is that we cannot reach all the nodes.
  So we can increase this number up to 2n-3+p and provide p>=0 possibility for retry in each loop and only ONE
  retry per node. It MUST be considered at planification time.
*/	
	if (status.retries < status.maxPerNodeRetries && t->hdr.retries < status.maxTotalRetries){/* set node to reached */
		status.retries++;
		t->hdr.retries++;
		return RETRY;
	}else{
		nstat_setReached(t->hdr.to);
		status.retries=0;
		rssi_reset(t->hdr.to);
		lqm_set_val(status.id,t->hdr.to,0);
		//		status.localLQM[status.id][(unsigned)t->hdr.to]=0;
		return EVALUATE_TOKEN;
	}
}

int manage_authorization_expired_timeout(wmpFrame * t){/* Authorization timeout expired*/
	if (status.retries < status.maxPerNodeRetries && t->hdr.retries < status.maxTotalRetries){/* set node to reached */
		status.retries++;
		t->hdr.retries++;
		return RETRY;
	}else{
		status.retries=0;
		rssi_reset(t->hdr.to);
		lqm_set_val(status.id,t->hdr.to,0);
		//status.localLQM[status.id][(unsigned)t->hdr.to]=0;		
		return NEW_TOKEN;
	}
}

int manage_message_expired_timeout(wmpFrame * t){/* Authorization timeout expired*/
	if (status.retries < status.maxPerNodeRetries && t->hdr.retries < status.maxTotalRetries){/* set node to reached */
		status.retries++;
		t->hdr.retries++;
		return RETRY;
	}else{
		status.retries=0;
		rssi_reset(t->hdr.to);
		lqm_set_val(status.id,t->hdr.to,0);
		//status.localLQM[status.id][(unsigned) t->hdr.to]=0;
		return NEW_TOKEN;
	}
} 

int evaluate_message(wmpFrame * t){
	EVENT_MESSAGE_RECEIVED(t);
	t->msg.age+=status.perHopDelay;
	if (t->msg.dest==status.id){//is for me
		Msg p;
		p.src=t->msg.src;
		p.len=t->msg.len;
		p.priority=t->msg.priority;
		p.dest=t->msg.dest;
		memcpy(&p.data[0],&t->msg.data,t->msg.len); /* ***TEMP Msg.len*/
		//p.protocol=0;
		p.port=t->msg.port;
		EVENT_PUSH_UC_MESSAGE(pushRcvdUCMsg(&p));
		EVENT_PUSHED_UC_MESSAGE(&p);
		status.acknow=p.src; //new	
		//if (status.UC_INTERRUPT) kill(0,SIGRTMAX-p.port);
		//printf("Esto es lo que escribo!!!!!!!!  %s\n",&p.data[0]);
		status.receivedMsg++;
		return NEW_TOKEN;
	} else{ /* Is not for me */
		int cost=0;
		int next=nextStepWithCost(lqm_get_ptr(),status.id,t->msg.dest,&cost);	
		if (next>=0){
			t->hdr.from=status.id;
			t->hdr.to=next;
			return SEND;
		}else{
			return NEW_TOKEN;
		}
	}
}

int evaluate_authorization(wmpFrame * t){
	EVENT_AUTHORIZATION_RECEIVED(t);
	if (t->aut.dest==status.id){/* Is for me */
		int ppp=getNumOfElementInTXUCQueue();
		if (ppp>0){			
			/* Pop the message */
			Msg j;
			long age=popUCMsgToSend(&j);	
			/* control if it exist a path to destination*/
			int cost=0;	
			int next=nextStepWithCost(lqm_get_ptr(),status.id,j.dest,&cost);
			if (next>=0){
				t->hdr.type=MESSAGE;
				t->hdr.from=status.id;				
				t->hdr.to=next;						
				t->msg.dest=j.dest;
				t->msg.src=status.id;
				t->msg.len=j.len;
				t->msg.port=j.port;
				t->msg.priority=j.priority;
				t->msg.age=age;
				memcpy(&t->msg.data,&j.data,j.len);
				if (t->msg.dest==status.id) return EVALUATE_MESSAGE; /*should not happen*/
				else return SEND;
			}else return NEW_TOKEN; /* NO PATH to destination */
		}else return NEW_TOKEN; /* NO waiting messages */
	}else{ /* The message is not for me */
		int cost=0;
		int next=nextStepWithCost(lqm_get_ptr(),status.id,t->aut.dest,&cost);
		if (next>=0){
			t->hdr.from=status.id;
			t->hdr.to=next;
			return SEND;
		} else return NEW_TOKEN; /* No path to destination*/
	}
}


